import os
import sys
import json

def getconfigfile():
    try:
        return json.load(open("./nephelo.json", "r"))
    except Exception as e:
        print "There was an error loading the config file. Expected at ./nephelo.json"
        print e
    return None




import compilescripts as Compiler
import os
import boto3
from netaddr import IPNetwork
import json
import math



def getsubnetsforregion(regionname):
    print regionname


def compile(libpath, config, stage):
    regions = config.get("regions")
    pathtocore = "./Infrastructure/Resources/Networking/"

    # need to work out the cidr blocks

    # how many networks are there? the top level CIDR block needs to be split across

    networks = []
    for level in os.walk(pathtocore):
        networks = level[1]
        break
    individual_networks = {}
    for network in networks:
        for region in regions:
            networkname = Compiler.deCamel(network)
            individual_networks[network + ":" + region] = {
                "vpc_name": "vpc" + stage + region.replace("-", "") + networkname
            }
    number_of_cidr_blocks = len(individual_networks.keys())
    subnets = splitCidrBlock(config.get("cidr_block"), number_of_cidr_blocks)
    print subnets
    for key in individual_networks:
        print subnets
        individual_networks[key]["cidr_block"] = subnets.pop(0)
    network_json = {}
    for region in regions:

        region_network_data = {}
        for network in networks:
            # create the vpc
            networkname = Compiler.deCamel(network)
            network_record = individual_networks[network + ":" + region]
            vpc = generateVpc(region, stage, network_record["vpc_name"], network_record["cidr_block"])
            # create the subnets
            subnets = generatesubnets(region, networkname, network_record["vpc_name"], stage, network_record["cidr_block"])
            subnet_ids = subnets.keys()
            subnet_ids = map(lambda x: "{\"Ref\":\"" + x + "\"}", subnet_ids)
            # compile the path
            config["region"] = region
            resources = Compiler.compilePath(libpath, pathtocore + network + "/", config, stage, {
                "vpc-id": network_record["vpc_name"],
                "subnet-ids": subnet_ids,
                "raw-subnet-ids": subnets.keys()
            })
            networkresources = {}
            for vpckey in vpc:
                networkresources[vpckey] = vpc[vpckey]
            for subnetkey in subnets:
                networkresources[subnetkey] = subnets[subnetkey]
            for resourcekey in resources:
                networkresources[resourcekey] = resources[resourcekey]
            region_network_data[network] = {
                "resources": networkresources,
                "cidr_block": network_record["cidr_block"],
                "subnet_ids": subnet_ids,
                "vpc_id": network_record["vpc_name"],
                "raw_subnet_ids": subnets.keys()
            }
        network_json[region] = region_network_data

    return network_json


def subnetids():
    return "\"\""


def generatesubnets(region, networkname, vpcid, stage, network_block):
    templates_dir = os.path.join(os.path.dirname(__file__), 'templates')
    subnet_template = templates_dir + "/subnet.json"
    zones = getZonesForRegion(region)
    subnet_blocks = splitCidrBlock(network_block, len(zones))
    subnets = {

    }
    for zone in zones:
        subnet_string = open(subnet_template).read()
        subnet_name = vpcid.replace("vpc", "subnet" + zone)
        cidr_block = subnet_blocks.pop(0)
        vpc_reference = "{\"Ref\":\"" + vpcid + "\"}"
        name = vpcid.replace("vpc", "subnet" + zone)
        subnet_string = subnet_string.replace("{SUBNETNAME}", subnet_name)
        subnet_string = subnet_string.replace("{NAME}", subnet_name)
        subnet_string = subnet_string.replace("{STAGE}", stage)
        subnet_string = subnet_string.replace("{CIDR_BLOCK}", cidr_block)
        subnet_string = subnet_string.replace("{AVAILABILITY_ZONE}", zone)
        subnet_string = subnet_string.replace("\"{VPC_REFERENCE}\"", vpc_reference)
        subnets[subnet_name.replace("-", "")] = json.loads(subnet_string)
    return subnets


def generateVpc(region, stage, vpc_name, cidr_block):
    templates_dir = os.path.join(os.path.dirname(__file__), 'templates')
    vpc_template = templates_dir + "/vpc.json"
    vpcstring = open(vpc_template).read()
    vpcstring = vpcstring.replace("{VPC_NAME}", vpc_name.replace("-", ""))
    vpcstring = vpcstring.replace("{NAME}", vpc_name)
    vpcstring = vpcstring.replace("{CIDR_BLOCK}", cidr_block)
    vpcstring = vpcstring.replace("{STAGE}", stage)
    return json.loads(vpcstring)


def getZonesForRegion(region):
    availZones = []
    ec2 = boto3.client('ec2', region_name=region)
    for zone in ec2.describe_availability_zones()['AvailabilityZones']:
        if zone['State'] == 'available':
            availZones.append(zone['ZoneName'])
    return availZones


def splitCidrBlock(cidr_block, number_of_cidr_blocks):
    net = IPNetwork(cidr_block)
    current_cidr_exponent = int(cidr_block.split("/")[-1])
    exponent_difference = math.log(number_of_cidr_blocks, 2)
    exponent_difference = math.ceil(exponent_difference)
    exponent_difference = int(exponent_difference)
    subnets = net.subnet(current_cidr_exponent + exponent_difference)
    subnets = list(subnets)
    if len(subnets) == 0:
        subnets.append(cidr_block)
    subnets = map(lambda x: str(x), subnets)
    print subnets
    return subnets


import json
import os
import sys
import segments as Segments


def saveTemplates(config, stage):
    config["stage"] = stage
    resources = Segments.getSegment("", "all", config, stage)
    regions = config.get("regions")
    if not os.path.exists("./dist"):
        os.makedirs("./dist")
    if not os.path.exists("./dist/" + stage + "/"):
        os.makedirs("./dist/" + stage + "/")
    for region in regions:
        filepath = "./dist/" + stage + "/" + region + ".json"
        region_json = resources[region]
        with open(filepath, "w") as text_file:
            text_file.write(json.dumps(region_json))
    return None


import os
import boto3

def deleteStack(config, stage):
    regions = config.get("regions")
    stack_name = stage + config.get("project")
    for region in regions:
        cloudformation = boto3.client('cloudformation', region_name=region)
        cloudformation.delete_stack(
            StackName=stack_name
        )
    return None


import config
import network
import segments as Segments
import template_generator as Templates
import deploy as Deployment
import delete as DeleteStack


def testpypi():
    return "This is packaged correctly"


def configure():
    return config.getconfigfile()


def get(filepath, config, segment, stage):
    print Segments.getSegment(filepath, segment, config, stage)
    return None


def save(config, stage):
    Templates.saveTemplates(config, stage)
    return None


def deploy(config, stage):
    save(config, stage)
    Deployment.deploy(config, stage)
    return ""


def deleteStack(config, stage):
    DeleteStack.deleteStack(config, stage)
    return None




import boto3
import os
s3 = boto3.client('s3', region_name="us-east-1")

def deploy(config, stage):
    regions = config.get("regions")
    stack_name = stage + config.get("project")
    bucket_name = config.get("bucket_name")

    # check if the bucket exists in the account and if it doesn't then create it
    getBucket(bucket_name)

    # transfer the distribution files to the bucket
    uploaded_files = uploadDistFiles(bucket_name, stack_name, stage)
    print uploaded_files
    stacks = []
    notification_arns = config.get("notification_arns", [])
    role_arn = config.get("iam_role")
    for region in regions:
        cloudformation = boto3.client('cloudformation', region_name=region)

        # check if the stack exists in this region
        disablerollback = config.get("disable_rollback", False)
        try:
            cloudformation.describe_stacks(
                StackName=stack_name
            )
            stack_details = cloudformation.update_stack(
                StackName=stack_name,
                TemplateURL="http://" + bucket_name + ".s3.amazonaws.com/" + stack_name + "/" + stage + "/" + region + ".json",
                NotificationARNs=notification_arns,
                RoleARN=iam_role
            )
            # The stack exists, we need to try and update the stack
            stacks.append(stack_details.get("StackId"))
        except Exception as e:
            print "CloudFormation Exception"
            print e
            # The stack doesn't exist, so we need to create it
            stack_details = cloudformation.create_stack(
                StackName=stack_name,
                TemplateURL="http://" + bucket_name + ".s3.amazonaws.com/" + stack_name + "/" + stage + "/" + region + ".json",
                DisableRollback=disablerollback,
                NotificationARNs=notification_arns,
                RoleARN=iam_role
            )
            stacks.append(stack_details.get("StackId"))


    return stack_details


def getBucket(bucket_name):
    try:
        s3.head_bucket(
            Bucket=bucket_name
        )
    except Exception as e:
        print "Exception"
        print e
        s3.create_bucket(
            ACL="private",
            Bucket=bucket_name
        )
    return bucket_name


def uploadDistFiles(bucket_name, stack_name, stage):
    files_to_upload = []
    for level in os.walk("./dist"):
        for filename in level[2]:
            filepath = level[0]
            if filepath[-1] != "/":
                filepath += "/"
            files_to_upload.append(filepath + filename)
    for file in files_to_upload:
        s3.upload_file(file, bucket_name, stack_name + "/" + file.replace("./dist/", ""))
    uploaded_files = map(lambda x: stack_name + "/" + x, files_to_upload)
    return uploaded_files



import network as Network
import compute as Compute
import compilescripts as Compiler
import json

networking_details = None
compute_details = None

def getSegment(filepath, segment, config, stage):
    networking_details = Network.compile(filepath, config, stage)
    if segment == "network":
        return networking_details
    elif segment == "compute":
        compute_details = Compute.compile(filepath, config, stage, networking_details)
        return compute_details
    elif segment == "all":
        compute_details = Compute.compile(filepath, config, stage, networking_details)
        config["stage"] = stage
        base = Compiler.loadfile(filepath, "./Infrastructure/base.json", config, {})
        all_segments = {}
        regions = config.get("regions")
        for region in regions:
            config["region"] = region
            all_segments[region] = {
                "Resources": {},
                "Mappings": {}
            }
            for key in networking_details[region]:
                network = networking_details[region][key]
                network_resources = network["resources"]
                # network_resources = json.loads(network_resources)
                for resource_key in network_resources:
                    all_segments[region]["Resources"][resource_key] = network_resources[resource_key]
            for key in compute_details[region]:
                network = compute_details[region][key]
                for resource_key in network:
                    all_segments[region]["Resources"][resource_key] = network[resource_key]
            for key in base:
                all_segments[region][key] = base[key]
            mappings = open('./Infrastructure/mappings.json', "r").read()
            mappings = json.loads(mappings)
            for key in mappings:
                all_segments[region]["Mappings"][key] = mappings[key]
        return all_segments
    return None











import compilescripts as Compiler
import os
import boto3
import json

def compile(libpath, config, stage, network_data):
    regions = config.get("regions")
    pathtocore = "./Infrastructure/Resources/Compute/"
    regional_compute = {}
    networks = []
    for level in os.walk(pathtocore):
        networks = level[1]
        break
    for region in regions:
        regional_compute_data = {}
        for network_name in networks:
            network = network_data[region][network_name]
            config["region"] = region
            resources = Compiler.compilePath(libpath, pathtocore, config, stage, {
                "vpc-id": network["vpc_id"],
                "subnet-ids": network["subnet_ids"],
                "raw-subnet-ids": network["raw_subnet_ids"]
            })
            regional_compute_data[network_name] = resources
        regional_compute[region] = regional_compute_data
    print regional_compute
    return regional_compute



from setuptools import setup

setup(name='nephelo',
      version='0.109',
      description='A Cloud Formation library to more easily. Nephelo, short for Nephelococcygia, is the art of watching clouds.',
      url='https://github.com/tamme-io/nephelo',
      author='tamme',
      author_email='opensource@tamme.io',
      license='MIT',
      packages=[
            'nephelo'
      ],
      install_requires=[
            'boto3',
            'netaddr'
      ],
      package_data={
            'nephelo': [
                  'templates/vpc.json',
                  'templates/subnet.json'
            ]
      },
      scripts=['bin/nephelo'],
      zip_safe=False)



import network as Network
import json
import os
import sys
import boto3
import re

accepted_config_keys = ["region", "cidr_prefix", "region_prefix", "subnet", "stage"]


def compilePath(libpath, path, config, stage, references):
    replacementdict = {
        "stage": stage
    }
    for key in config:
        if key in accepted_config_keys:
            replacementdict[key] = config[key]
    files = []
    print path
    for step in os.walk(path):
        print step
        dirpath = step[0]
        if dirpath != "/":
            dirpath += "/"
        for filename in step[2]:
            files.append(dirpath + filename)
    files = filter(lambda x: ".json" in x, files)
    jsonobjects = []
    for filepath in files:
        try:
            jsonobjects.append(loadfile(libpath, filepath, replacementdict, references))
        except Exception as e:
            print e
    compiledresources = {}
    for jsonobject in jsonobjects:
        for key in jsonobject:
            compiledresources[key] = jsonobject[key]
    return compiledresources


def loadfile(libpath, path, replacementdict, reference_dict):
    try:
        filestring = open(path).read()
        # print filestring
        filename = path.split("/")[-1]
        for key in replacementdict:
            if key not in accepted_config_keys:
                continue
            # find the iterators
            iterator_start = "%{"
            iterator_end = "}%"
            # print "Regex"
            # print iterator_start + '(.*?)' + iterator_end
            iterators = re.findall(iterator_start + '(.*?)' + iterator_end, filestring, re.DOTALL)
            # print iterators
            for iterator in iterators:
                # what are we iterating and what is the local variable?
                # print "THIS HERE"
                iteration_object = iterator.split(" as ")[0].replace(" ", "")
                # print iteration_object
                iteration_name = iterator.split(" as ")[1].split("|,")[0].replace("|", "").replace(" ", "")
                iteration_template = iterator.split(" as ")[1].split("|,")[1]
                # print iteration_template
                if iteration_object in replacementdict or iteration_object in reference_dict:
                    iteration_list = reference_dict[iteration_object] if iteration_object in reference_dict else replacementdict[iteration_object]
                    # print iteration_list
                    if isinstance(iteration_list, (list, tuple)):
                        results = []
                        for item in iteration_list:
                            replacestring = "${" + iteration_name + "}"
                            replacevalue = item if isinstance(item, str) else str(item)
                            iteration_result = iteration_template.replace(replacestring, replacevalue)
                            replacestring = "@{" + iteration_name + "}"
                            replacevalue = item if isinstance(item, str) else str(item)
                            iteration_result = iteration_result.replace(replacestring, "{\"Ref\": \"" + replacevalue + "\"}")
                            results.append(iteration_result)
                        resultstring = "".join(results)
                        # print resultstring
                        filestring = filestring.replace(iterator_start + iterator + iterator_end, resultstring)
                    else:
                        filestring = filestring.replace(iterator_start + iterator + iterator_end, "")
                else:
                    filestring = filestring.replace(iterator_start + iterator + iterator_end, "")


            filestring = filestring.replace("${" + key + "}", replacementdict[key])
            if 'subnet-ids' in reference_dict:
                subnet_ids = map(lambda x: json.loads(x), reference_dict["subnet-ids"])
                filestring = filestring.replace("!{subnets}", json.dumps(subnet_ids))
                for index in range(len(reference_dict['subnet-ids'])):
                    filestring = filestring.replace("!{subnets[" + str(index) + "]}", reference_dict["subnet-ids"][index])
            env_start = "@@{"
            env_end = "}@@"
            env_var_refs = re.findall(env_start + '(.*?)' + env_end, filestring, re.DOTALL)
            print env_var_refs
            for env_ref in env_var_refs:
                replacement_value = os.environ.get(env_ref, "")
                filestring = filestring.replace(env_start + env_ref + env_end, replacement_value)
            start = "@{"
            end = "}"
            # print "Reference Regex"
            # print start + '(.*?)' + end
            references = re.findall(start + '(.*?)' + end, filestring, re.DOTALL)
            # print references
            for reference in references:
                if reference in reference_dict:
                    filestring = filestring.replace("@{" + reference + "}", "{ \"Ref\":\"" + reference_dict.get(reference) + "\"}")
                else:
                    filestring = filestring.replace("@{" + reference + "}", "{ \"Ref\":\"" + reference + "\"}")

        return json.loads(filestring)
    except Exception as e:
        print filestring
        print reference_dict
        print e
    return {}


def deCamel(name):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1\2', s1).lower()